
% PRCTILE_MLX Percentiles of a sample.
%
% y = prctile_mlx(x, p)
% y = prctile_mlx(x, p, dim)
%
% Computes the value associated with the p-th percentile of x. If x is a
% matrix, computes p for each column of x.  The returned
% value is a matrix with one row for each element of p and one column for
% each column of x. If x is a vector and dim is not given, then y is a
% vector with the same size as p. If the argument dim is given, operate
% along this dimension rather than the columns of the matrix.
%
% The first and last values are pegged at 0 percent and 100 percent
% respectively, and the rest of the values are uniformly spaced between
% them, with linear interpolation between the points.

function a = prctile_mlx(x,p,dim)

% check percentile
if isempty( p ) || ~isvector( p )
	error( 'MLX:PRCTILE_MLX:BadPercents', ...
		'p must be a scalar or a non-empty vector' );
elseif any( p < 0 | 100 < p )
	error( 'MLX:PRCTILE_MLX:BadPercents', ...
		'p must be between 0 and 100' );
end

dimFlag = true;
if nargin == 2
	dimFlag = false;
	
	dim = find( size(x)~= 1, 1 );
	if isempty( dim )
		dim = 1;
	end
	
end

if isempty(x)
	if all( size( x ) == [0 0] ) && ~dimFlag
		a = nan( size(p) );
	else
		sx = size( x ); sx( dim ) = numel( p );
		a = nan( sx );
	end
	return
end

perm = [ dim:max(ndims(x),dim) 1:dim-1];

sx = size( x ); sx( length( sx )+1:dim ) = 1;
x = permute( x, perm );
x = reshape( x, size(x,1), [] );
x = sort( x, 1 );

nbCol = size( x, 2 );
a = zeros( numel( p ), nbCol );
if ~any( isnan( x ) )
	
	lx = sx( dim );
	a = interp1q( [ 0 100*(0.5:(lx-0.5))/lx 100 ]', [ x(1,:);x;x(end,:) ], p(:) );

else
	
	for i = 1:nbCol
		
		xx = x(:,i);
		xx = xx( ~isnan( xx ) );
		if isempty( xx )
			a(:,i) = nan( size( p ) );
		else
			lx = length( xx );
			a(:,i) = interp1q( [ 0 100*(0.5:(lx-0.5))/lx 100 ]', [ xx(1);xx;xx(end) ], p(:) );
		end
		
	end
	
end

if dimFlag
	a = ipermute( a, perm );
	sx( dim ) = numel( p );
	a = reshape( a, sx );
elseif isvector( x )
	a = reshape( a, size(p) );
end

end
