function [admin, nbInds]= processing_administration(admin)
%processing_administration 
%
%   [admin, nbInds] = processing_administration(admin)
%       return admin struct and the number of differents individuals
%       dealing with the structure
%            Set the good format for each field considering time and id
%            field
%        
%
%   Examples
%   --------
%     [adminOut, nbInds] = processing_administration(admin)
% 
%     admin{1} =
% 
%         target: 'Ad'
%         amount: 1
%           time: [6 30 54 78]
% 
%     admin{2} =
% 
%         target: 'Ac'
%         amount: 1
%           time: [12 36 60 84]
%           rate: 0.1000
%
%       nbInds   = 1
%       adminOut = 
% 
%             id: 1
%           time: {[6 30 54 78 12 36 60 84]}
%         amount: {[1 1 1 1 1 1 1 1]}
%           rate: {[0 0 0 0 0.1000 0.1000 0.1000 0.1000]}
%           type: {[1 1 1 1 2 2 2 2]}
%          label: {'none'}
%           name: {'none'}
%         target: {'Ad'  'Ad'  'Ad'  'Ad'  'Ac'  'Ac'  'Ac'  'Ac'}




nb_admin= length(admin);
% format any single admin cell
admin = cellfun(@formatAdmin,admin,'UniformOutput',false);

%% MANAGE DEPOTS
k=1;
for i=1:nb_admin
    if isfield(admin{i},'target')
        for (j=1 : length(admin{i}.id))
           admin{i}.type{j} = admin{i}.type{j}*k; % a changer 
        end
        k=k+1;
    end
end

%% get the number of individuals over all admin cell
nbInds=zeros(1,length(admin));
for i=1:nb_admin
    nbInds(i)=length(unique(admin{i}.id));
end
nbInds=max(nbInds);


%% CONCATENATION WHEN MANY ADMINISTRATIONS ARE COMBINED
admin = combinedAdministration(admin);


function admin = combinedAdministration(admin)
%combinedAdministration 
%
%   admin = combinedAdministration(admin)
%       return admin struct which is the concatenation of every cell of
%       admin
%        
%
%   Examples
%   --------
%     adminOut = combinedAdministration(admin)
% 
%     admin{1} =
% 
%         target: {'Ad'  'Ad'  'Ad'  'Ad'}
%         amount: {[1 1 1 1]}
%           time: {[6 30 54 78]}
%             id: 1
%           name: {'none'  'none'  'none'  'none'}
%          label: {'none'  'none'  'none'  'none'}
%           type: {[1 1 1 1]}
%           rate: {[0 0 0 0]}
% 
% 
% 
%     admin{2} =
% 
%         target: {'Ac'  'Ac'  'Ac'  'Ac'}
%         amount: {[1 1 1 1]}
%           time: {[12 36 60 84]}
%           rate: {[0.1000 0.1000 0.1000 0.1000]}
%             id: 1
%           name: {'none'  'none'  'none'  'none'}
%          label: {'none'  'none'  'none'  'none'}
%           type: {[2 2 2 2]}
% 
%     adminOut =
%             id: 1
%           time: {[6 30 54 78 12 36 60 84]}
%         amount: {[1 1 1 1 1 1 1 1]}
%           rate: {[0 0 0 0 0.1000 0.1000 0.1000 0.1000]}
%           type: {[1 1 1 1 2 2 2 2]}
%          label: {'none'}
%           name: {'none'}
%         target: {'Ad'  'Ad'  'Ad'  'Ad'  'Ac'  'Ac'  'Ac'  'Ac'}


  
nb_admin= length(admin);
if nb_admin>1
    % create admin struct from admin cell
    design = struct();
    % initialization
    design.id     = [] ;
    design.time   = [] ;
    design.amount = [] ;
    design.rate   = [] ;
    design.type   = [] ;
    design.label  = {} ;
    design.name   = {} ;
    if isfield(admin{1},'target')  %isfiedlAtLeastOne()
        design.target = {} ;
        if isfield(admin{1},'tlag') %isfiedlAtLeastOne()
            design.tlag = [];
        end
        if isfield(admin{1},'p')  %isfiedlAtLeastOne()
            design.p = [];
        end
    end

    % concatenate all admin fields
    for i=1:nb_admin
        design.id     = [design.id     admin{i}.id    ];
        design.time   = [design.time   admin{i}.time{1}  ];
        design.amount = [design.amount admin{i}.amount{1} ];
        design.rate   = [design.rate   admin{i}.rate{1}  ]; 
        design.type   = [design.type   admin{i}.type{1}   ];
        design.label  =  horzcat(design.label,  tools.tocell(admin{i}.label));        
        design.name   =  horzcat(design.name,  tools.tocell(admin{i}.name));
        if isfield(admin{i},'target')
            design.target = horzcat(design.target,  tools.tocell(admin{i}.target));
            if isfield(admin{i},'tlag')
                design.tlag   =  [design.tlag   admin{i}.tlag   ];
            end
            if isfield(admin{i},'p')
                design.p      =  [design.p   admin{i}.p   ];
            end
        end          
    end
    design.id=unique(design.id);

    % modificatiion of the structure format
    admin=struct();
    admin.id     = design.id;
    admin.time   = tools.tocell(design.time);
    admin.amount = tools.tocell(design.amount);
    admin.rate   = tools.tocell(design.rate);
    admin.type   = tools.tocell(design.type);
    admin.label  = tools.tocell(unique(design.label));
    admin.name   = tools.tocell(unique(design.name));  
    if isfield(design,'target')
       admin.target   = tools.tocell(design.target);
       if isfield(design,'tlag')
           admin.tlag =  tools.tocell(design.tlag);
       end
       if isfield(design,'p')
           admin.p    =  tools.tocell(design.p);
       end
    end    
else
    admin = admin{1};
end

function admin = formatAdmin(admin)
%formatAdmin 
%
%   admin = formatAdmin(admin)
%       checks fields in admin structure and formats it by setting default field value
%            Set the good global format and settle each field considering time and id
%            field

%   Examples 1 
%   --------
%       adminOut = formatAdmin(admin)
% 
%       admin = 
%           amount: 1
%           time: 2
%       
%       adminOut = 
% 
%         amount: {[1]}
%           time: {[2]}
%             id: 1
%           name: 'none'
%          label: 'none'
%           type: {[1]}
%           rate: {[0]}
%
%
%   Examples 2 
%   --------
%     admin{1} =
% 
%         target: 'Ad'
%         amount: 1
%           time: [6 30 54 78]
% 
%     admin{2} =
% 
%         target: 'Ac'
%         amount: 1
%           time: [12 36 60 84]
%           rate: 0.1000
% 
%     adminOut{1} =
% 
%         target: {'Ad'  'Ad'  'Ad'  'Ad'}
%         amount: {[1 1 1 1]}
%           time: {[6 30 54 78]}
%             id: 1
%           name: {'none'  'none'  'none'  'none'}
%          label: {'none'  'none'  'none'  'none'}
%           type: {[1 1 1 1]}
%           rate: {[0 0 0 0]}
% 
% 
% 
%     adminOut{2} =
% 
%         target: {'Ac'  'Ac'  'Ac'  'Ac'}
%         amount: {[1 1 1 1]}
%           time: {[12 36 60 84]}
%           rate: {[0.1000 0.1000 0.1000 0.1000]}
%             id: 1
%           name: {'none'  'none'  'none'  'none'}
%          label: {'none'  'none'  'none'  'none'}
%           type: {[2 2 2 2]}
      

%% get all default field values
defaultID      = tools.defaultFieldValue('id'); 
defaultNAME    = tools.defaultFieldValue('name');
defaultLABEL   = tools.defaultFieldValue('label');
defaultRATE    = tools.defaultFieldValue('rate');
defaultTYPE    = tools.defaultFieldValue('type');
defaultAMOUNT  = tools.defaultFieldValue('amount'); 

if isstruct(admin)
    hasVALUE  = isfield(admin,'value');
    hasHEADER = isfield(admin,'header');
    hasTYPE   = isfield(admin,'type');
    
    %% if header and value field are both given, we format the structure according to header.
    if ~hasVALUE && hasHEADER
        error('admin with a header must have value as a field');
    elseif ~hasHEADER && hasVALUE
        error('admin with a value must have header as a field');
    elseif hasHEADER && hasVALUE
        admin = tools.addFieldsFromHeader(admin);
    end
    
    
    %% An administration has to have "time as a field" 
    hasTIME   = isfield(admin,'time');
    if hasTIME
        hasTARGET = isfield(admin,'target');
        if hasTARGET
            if hasTYPE
                error('admin with a target should not have type as a field');
            end
        end
        % create default field with default value if the field does not exist         
        admin = tools.setNonExistentField(admin,'id',defaultID);
        admin = tools.setNonExistentField(admin,'name',defaultNAME);
        admin = tools.setNonExistentField(admin,'label',defaultLABEL);
        admin = tools.setNonExistentField(admin,'amount',defaultAMOUNT);
        admin = tools.setNonExistentField(admin,'type',defaultTYPE);
        admin = tools.setNonExistentField(admin,'rate',defaultRATE);
        
        % check that the number of field in admin structure is correct. 
        % It should not be up to nbFields
        % Expected field: name, id, type, amount, rate, time, label,
        % target, tlag et p

        nbFields = 10;
        if length(fieldnames(admin)) > nbFields
            error('presence of unexpected fields in admin');
        end 
        
        % get id dimension
        dimID=length(admin.id);
        
        % get time dimension
        if iscell(admin.time)
             dimTIME= ones(1, dimID);
             for i=1:length(admin.time)
                 dimTIME(i) =  length(admin.time{i});
             end
        else
            dimTIME(1)      = length(admin.time);
        end

        
        
         %% processing of numerical fields
        if isnumeric(admin.time)
            admin.time  = tools.tocell(admin.time);
        end
        
        if isnumeric(admin.amount)
            admin = proc_numerical(admin, 'amount', dimID, dimTIME);
        end
        if isnumeric(admin.type)
            admin = proc_numerical(admin, 'type', dimID, dimTIME);
        end
        if isnumeric(admin.rate)
            admin = proc_numerical(admin, 'rate', dimID, dimTIME);
        end

            
         %% processing of textual fields ( mandatory for concaténation of administartions in processing_administration )
         if length(dimTIME)==1
            admin.name   = tools.replicateCellField(admin.name,dimTIME(1));
            admin.label  = tools.replicateCellField(admin.label,dimTIME(1));
         else
            admin.name   = tools.replicateCellField(admin.name,dimID);
            admin.label  = tools.replicateCellField(admin.label,dimID);
         end

                    
         %% processing of the depot field
         if hasTARGET
             admin.target  = tools.replicateCellField(admin.target,dimTIME(1));
             if isfield(admin,'tlag')
                 admin.tlag = tools.tovector(admin.tlag,dimTIME(1));
             end
             if isfield(admin,'p')
                 admin.p    = tools.tovector(admin.p,dimTIME(1));
             end
         end  
    else
        error('admin must have time as a field');      
    end     
else
    error('admin must be a structure');
end


function structure = proc_numerical(structure, field, dimID, dimTIME)

%% processing of numerical field if it needs 
val = structure.(field);
structure.(field)={};
for i=1 : dimID  
    structure.(field){i,1}  = tools.tovector(val,dimTIME(i));
end
%% create an identical cell per individual
structure.(field)  = tools.replicateCellField(structure.(field),dimID);



