function group = processing_group(group,admin, output,paramInd,nb_group )
%processing_group 
%
%    group = processing_group(param)
%       handles group according to wether it exist or not.
%       return a new structure group or the modified input one if already
%       exist.
%        


if ~isempty(group)
    %if group exists, then it is formatted
    group = cellfun(@(x) formatGroup(x,admin,output),group,'UniformOutput',false);
else
    %if group does not exist, then it is created
    group    = createGroup(admin, paramInd, output, nb_group);
end


function group = formatGroup(group,admin,output)
%formatGroup 
%
%   group = formatGroup(group,admin,output)
%       checks and format structure admin, parameter and output structures in group 
%            Set admin and output if they do not exist in group    
%        
%   Examples 1 
%   --------
%        groupOut = formatGroup(group,admin,output))
% 

% group{1} =
%  
%               size: 2
%     treatment: [1x1 struct]
%              level: 'individual'
% 
%  
%  
% group{2} =
%  
%               size: 3
%     treatment: [1x1 struct]
%              level: 'individual'
%              
% groupOut = 
% 
%                  size: 2
%             treatment: [1x1 struct]
%             parameter: [1x1 struct]
%                 level: 'individual'
%            parameters: [1x1 struct]
%     output_individual: [1x1 struct]
%          output_group: [1x1 struct]

             
isADMIN  = ~isempty(admin);
isOUTPUT = ~isempty(output);

if isstruct(group)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %           Manage inputs
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    hasADMIN  = isfield(group,'treatment');
    hasPARAM  = isfield(group,'parameter');
    hasOUTPUT = isfield(group,'output');

    if isADMIN && hasADMIN
        error('treatment exists but group contains administration');  
    elseif isOUTPUT && hasOUTPUT
        error('output exists but group contains output');
    elseif ~isOUTPUT && ~hasOUTPUT
        error('output must be provided');   
    %
    else
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %           manage designs
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if isADMIN | hasADMIN
            if hasADMIN
                [admin, nbIndsAdmin]= processing_administration(tools.tocell(group.treatment));
                if nbIndsAdmin~=1
                    error('treatment format is incorrect inside a group');
                end
            end
                idx=1;  
                group = adminToGroup(group, admin, idx);
        end

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %           manage parameters
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if hasPARAM
            [temp, singleParam, nbIndsParam]= processing_parameter(tools.tocell(group.parameter));
            if nbIndsParam~=1
                error('parameter format is incorrect inside a group');
            end

            if ~isempty(singleParam)
                group.parameters.name     = singleParam.name;
                %group.parameters.label   = paramPop.label;
                group.parameters.value    = singleParam.value;
            end
        end

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %           manage output
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%s%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if hasOUTPUT 
            [outputIndividual, outputGroup, nbIndsOutput]= processing_output(tools.tocell(group.output));
            if nbIndsOutput~=1
                error('output format is incorrect inside a group');
            end
            output            = struct();
            output.individual = outputIndividual;
            output.group      = outputGroup;
        end
        if ~isempty(output.individual)
            idx=1;
            group = outputToGroup( group, output, idx);
        else
            error('individual output should not be empty');
        end
        if ~isempty(output.group)
            idx=1;              
            group.output_group.name   = horzcat(group.output_group.name ,output.group.name{idx}); 
            group.output_group.label  = horzcat(group.output_group.label ,output.group.label{idx});
        end

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %           manage size et level
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if ~isfield(group, 'size')
            group.size  = 1;
        end
        if ~isfield(group, 'level')
            group.level  = 'longitudinal';
        end
    end
else
    error('group must be a struct');
end

function group = createGroup(admin, paramInd, output, nb_group)
%createGroup 
%
%   group = createGroup(admin, paramInd, output, nb_group)
%       create a group based on paramInd and output structure and the
%       number of group nb_group  
%        
%   Examples 1 : group does not exist 
%   --------
%        group = formatGroup(group,admin,output)
% 
%      group = 
% 
%      {}
% 
% admin = 
% 
%     amount: {[1]}
%       time: {[2]}
%         id: 1
%       name: {'none'}
%      label: {'none'}
%       type: {[1]}
%       rate: {[0]}
%
% paramInd  = 
%      name: {{1x1 cell}}
%     label: {{1x1 cell}}
%      time: {{1x1 cell}}
%        id: 1
%
% nb_group =
% 
%      1
% 
% group{1}
% 
%                      size: 1
%                     level: {'longitudinal'}
%            treatment: [1x1 struct]
%         output_individual: [1x1 struct]
%              output_group: [1x1 struct]
             

group=cell(1,nb_group);
for i=1 : nb_group
        group{i}.size      = 1;
        group{i}.level     = tools.tocell('longitudinal');
        

 %% manage individual parameters in group structure       
        if ~isempty(paramInd)
            idx=find(paramInd.id==i);
            if ~isempty(idx)
                group{i}.parameters.name    = paramInd.name{idx};
                group{i}.parameters.label   = paramInd.label{idx};
                group{i}.parameters.value   = paramInd.value{idx};
            end
        end
 %% manage treatment in group structure      
        if ~isempty(admin)
            idx=find(admin.id==i);
            if isempty(idx)
                idx=1;
            end  
            group{i} = adminToGroup( group{i}, admin, idx);
        end
        
 %% manage output in group structure 
        if ~isempty(output.individual)
            idx=find(output.individual.id==i);
            if isempty(idx)
                idx=1;
            end   
            group{i} = outputToGroup( group{i}, output, idx);
        else
            error('individual output should not be empty');
        end
        if ~isempty(output.group)
            idx=find(output.group.id==i);
            if isempty(idx)
                idx=1;
            end                
            group{i}.output_group.name   = horzcat(group{i}.output_group.name ,output.group.name{idx}); 
            group{i}.output_group.label  = horzcat(group{i}.output_group.label ,output.group.label{idx});
        end
end   

function group = outputToGroup( group, output, idx)
group.output_individual=struct();
group.output_group=struct();
%  individual field
group.output_individual.time    = output.individual.time{idx};
group.output_individual.name    = output.individual.name{idx}; 
group.output_individual.label   = output.individual.label{idx};  
%  group field
group.output_group.name         = output.individual.name{idx}; 
group.output_group.label        = output.individual.label{idx};

function group = adminToGroup(group, admin, idx)
group.treatment=struct();
group.treatment.time       = admin.time{idx};
group.treatment.amount     = admin.amount{idx};
group.treatment.type= admin.type{idx};
if isfield(admin,'target')
    group.treatment.target = admin.target; 
    if isfield(admin,'tlag')
        group.treatment.tlag = admin.tlag{idx};  
    end
    if isfield(admin,'p')
        group.treatment.p    = admin.p{idx};  
    end
end

group.treatment.rate       = admin.rate{idx};
group.treatment.name       = admin.name{idx}; 
group.treatment.label      = admin.label{idx};

