function [model, admin, param, output] = processing_monolix(project,model, admin, param, output )
%%% processing_monolix
%
%  [model, admin, param, output] = processing_monolix(project,model, admin, param, output )
%     takes a monolix project and extract information from
%     mlxtran file such as model, admin, param, output, if
%     they missed in the input parameters. Theses informations are read in
%     files.
%
%     Exemple
%     --------
%     [modelOut, adminOut, paramOut, outputOut] = processing_monolix(project,model, admin, param, output )
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                INPUT PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     project =
%
%         mlxtranfile: 'warf2_project.mlxtran'
%             session: [1x1 struct]
%
%     model =
%
%     warfarin1_model.txt
%
%     admin =
%
%          {}
%
%     param =
%
%          {}
%
%     output =
%
%          {}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                OUTPUT PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%     modelOut =
%
%     warfarin1_model.txt
%
%     adminOut
%
%     admin{1} =
%
%          label: 'source'
%           name: 'doseRegimen'
%         header: {'id'  'time'  'amount'}
%          value: [32x3 double]
%
%     paramOut
%
%     param{1} =
%
%           name: 'wt'
%          label: 'covariate'
%         header: {'id'  'wt'}
%          value: [32x2 double]
%
%
%     param{2} =
%
%          name: {1x21 cell}
%         value: [1x21 double]
%
%     outputOut
%
%     output{1} =
%
%          label: 'observation'
%           name: 'conc'
%         header: {'id'  'time'}
%          value: [251x2 double]
%
%
%
%     output{2} =
%
%          label: 'observation'
%           name: 'pca'
%         header: {'id'  'time'}
%          value: [232x2 double]


%%************************************************************************
%       XML FILE
%*************************************************************************
infoProject = getInfoXml(project.mlxtranfile);

%%************************************************************************
%       DATA FILE
%**************************************************************************
datas = readdatamlx(infoProject);

%%************************************************************************
%       ADMIN (TREATMENT)
%**************************************************************************
if isempty(admin) && isfield(datas,'source')
    admin = datas.source;
end

%%************************************************************************
%       PARAMETERS
%**************************************************************************
if isfield(datas,'covariate')
    ind_param      = datas.covariate;
else
    ind_param={};
end
nb_param           = length(ind_param);
paramp             = ind_param;
pop_param          = readPopEstimate([infoProject.resultFolder,'/estimates.txt']);
paramp{nb_param+1} = pop_param;
iop_indiv          = 0;
if isempty(param)
    param = paramp;
else
    if iscell(param{1}) || isstruct(param{1})
        param=mergeArg(paramp,param);
    else
        file       = [infoProject.resultFolder,'/indiv_parameters.txt'];
        param      = readIndEstimate(file,param);
        iop_indiv  = 1;
    end
end

%%************************************************************************
%       OUTPUT
%**************************************************************************

outputp = datas.observation;
if isempty(output)
    output = outputp;
else
    output = mergeArg(outputp,output);
end

%%************************************************************************
%       MODEL
%**************************************************************************
if isempty(model)
    [ppath,pname,pext]= fileparts(project.mlxtranfile);
    if isempty(ppath)
        ppath='.';
    end
    model = fullfile(ppath,[pname,'_model.txt']);
    session  = getenv('sessionSimulx');
    zz = fullfile(session,'lib','lixoftLanguageTranslator');
    str=['"',zz,'" --from=mlxproject --to=mlxtran --output-file=',model];
    str=[str,' --input-file=',project.mlxtranfile,' --option=with-observation-model'];
    system(str) ;
    
    if iop_indiv
        
        % reate a submodel file of model_file corresponding to the specified sections specified
        section       = 'LONGITUDINAL'   ;
        myparseModel(model, section, model );
        inputList    = getInputSection(model, section);
        var_m        = setdiff(inputList, param{1}.name);
        
        name  = {};
        value = [];
        for (i = 1 : length(var_m))
            idx   = find(strcmp(pop_param.name,var_m{i}));
            name  = [name, pop_param.name{idx}];
            value = [value, pop_param.value(idx)];
        end
        nb_param = length(param);
        param{nb_param+1}.name = name;
        param{nb_param+1}.value = value;
        %  else
        %      system([pathLib '/lib/mlxExport --with-observation-model --export-only-model=' model ' --project=' project.mlxtranfile]);
    end
end


function [infoProject] = getInfoXml(project)
%%% getInfoXml
%
%  [infoProject] = getInfoXml(project)
%     return infoProject which contains informations about the given mlxtran project
%
%     Exemple
%     --------
%     [infoProject] = getInfoXml(project)
%     project =
%           warf2_project.mlxtran
%
%     infoProject =
%           resultFolder     : './warf2_project'
%           datafile         : './warfarin_data.txt'
%           dataformat       : '\t'
%           dataheader       : {'ID'  'TIME'  'AMT'  'Y'  'YTYPE'  'COV'  'IGNORE'  'IGNORE'}
%           output           : {'conc'  'pca'}
%*****************************************************************************************

infoProject = struct();
%%************************************************************************
%       XML FILE
%*************************************************************************
% get path and name of monolix project
[mlxtranpath,mlxtranfile,ext] = fileparts(project);
if isempty(mlxtranpath)
    mlxtranpath='.';
end
infoProject.mlxtranpath =mlxtranpath;

% create  .xmlx from .mlxtran if necessary
if strcmp(ext, '.mlxtran')
    session= getenv('sessionSimulx');
    tmp = fullfile(mlxtranpath,'tmp987654321');
    xmlfile=fullfile(mlxtranpath,[mlxtranfile,'_tr.xmlx']);
    zz=fullfile(session,'lib','lixoftLanguageTranslator');
    str=['"',zz,'" --from=mlxproject --to=xmlx'];
    str=[str,' --output-file=',xmlfile,' --input-file=',project,' --option=with-observation-model'];
    system(str);
else
    xmlfile = project;
end


%%************************************************************************
%       RESULT FOLDER
%*************************************************************************
infoResultFolder             = myparseXML(xmlfile,'resultFolder',mlxtranpath);
infoProject.resultFolder     = infoResultFolder{1}.uri;

%%************************************************************************
%       DATA INFO
%*************************************************************************
%  get data format and data header used in the current project
%   Exemple :
%
%   infoProject =
%           datafile         : './warfarin_data.txt'
%           dataformat       : '\t'
%           dataheader       : {'ID'  'TIME'  'AMT'  'Y'  'YTYPE'  'COV'  'IGNORE'  'IGNORE'}
%
infoData                 = myparseXML(xmlfile,'data',mlxtranpath);
infoProject.datafile     = infoData{1}.uri;
if isfield(infoData{1},'columnDelimiter')
    infoProject.dataformat = infoData{1}.columnDelimiter;
end
infoProject.dataheader   = tools.strsplit(infoData{1}.headers, ',');

%%************************************************************************
%       OUTPUT INFO
%*************************************************************************

infoOutput = myparseXML(xmlfile,'observationModel',mlxtranpath);
for k=1:length(infoOutput)
    infoProject.output{k} = infoOutput{k}.name;
end

delete(xmlfile)


function s = myparseXML(filename,NodeList,mlxtranpath)
% MYPARSEXML Convert a selected list of attributes of a XML file to a MATLAB structure.
try
    tree = xmlread(filename);
catch
    error('Failed to read XML file %s.',filename);
end

% Recurse over child nodes.
res=cell(1,0);
res = parseChildNodes(tree,NodeList,res);
nres=length(res);
s=cell(1,nres);
sList=cell(1,nres);
for i=1:nres
    ri=res{i};
    nri=length(ri);
    si = struct('name',ri(1).parent);
    sList{i}=si.name;
    for j=1:nri
        rij=ri(j);
        si=setfield(si,rij.Name,rij.Value);
    end
    s{i}=si;
end

for i=1:length(s)
    if isfield(s{i},'uri')
        [upath,uname,uext]=fileparts(s{i}.uri);
        upath = strrep(upath,'%MLXPROJECT%',mlxtranpath);
        s{i}.uri=fullfile(upath,[uname,uext]);
    end
end

% ----- Subfunction PARSECHILDNODES -----
function res = parseChildNodes(theNode,NodeList,res)
% Recurse over node children.
if theNode.hasChildNodes
    childNodes = theNode.getChildNodes;
    numChildNodes = childNodes.getLength;
    
    for count = 1:numChildNodes
        theChild = childNodes.item(count-1);
        [a,res]=makeStructFromNode(theChild,NodeList,res);
        if ~isempty(a)
            k=length(res)+1;
            res{k}=a;
        end
    end
end

% ----- Subfunction MAKESTRUCTFROMNODE -----
function  [a,res]=makeStructFromNode(theNode,NodeList,res)
% Create struc'''ture of node info.
a=parseAttributes(theNode,NodeList);
res=parseChildNodes(theNode,NodeList,res);

% ----- Subfunction PARSEATTRIBUTES -----
function attributes = parseAttributes(theNode,NodeList)
% Create attributes structure.
attributes = [];
if theNode.hasAttributes
    NodeName=char(theNode.getNodeName);
    if any(strcmp(NodeName,NodeList))
        %    if strcmp(NodeName,'data') || strcmp(NodeName,'resultFolder')
        theAttributes = theNode.getAttributes;
        numAttributes = theAttributes.getLength;
        allocCell = cell(1, numAttributes);
        attributes = struct('Name', allocCell, 'Value', ...
            allocCell);
        
        for count = 1:numAttributes
            attrib = theAttributes.item(count-1);
            attributes(count).Name = char(attrib.getName);
            attributes(count).Value = char(attrib.getValue);
            attributes(count).parent=NodeName;
        end
    end
end

function param = readPopEstimate(filename)
%readPopEstimate
%
%  param = readPopEstimate(filename)
%       get the population parameter situated in filename file
%

% opening file
fid=fopen(filename);
cpt=0;
param.name={};
param.value=[];
if fid==-1
    error(['can not open ',filename, ' file'])
    return
end
while 1
    line = fgetl(fid);
    if ~ischar(line)
        % end of file : we stop reading
        break
    end
    C = tools.strsplit(line,';');
    if cpt==0
        for i=1 : length(C)
            if strcmp(C{i}, 'parameter')
                cpt = 1;
                break;
            end
        end
    else
        c=C{1};
        if length(c)>9 && strcmp(c(1:5),'corr(')
            i1=strfind(c,',');
            i2=strfind(c,')');
            name={c(6:i1(1)-1),c(i1(1)+1:i2(1)-1)};
 %           name=sort(name);
            c=['r_',name{1},'_',name{2}];
        end
        v=C{i};
        param.name =  horzcat(param.name, c);
        %horzcat(design.name,  tocell(admin{i}.name));
        param.value = [ param.value str2num(v)];
        cpt=cpt+1;
    end
    
    
end

fclose(fid);

function param = readIndEstimate(filename,estim)
%readIndEstimate
%
%  param = readIndEstimate(filename,estim)
%       get the individual parameter situated in filename file
%

fid=fopen(filename);
if fid==-1
    error(['can not open ',filename, ' file'])
    return
end
line = fgetl(fid);
C = tools.strsplit(line,' ');
j=0;
header={'id'};
cj=zeros(1,0);
stropt=['_',estim{1}];
for k=1:length(C)
    ik=regexp(C{k},stropt);
    if ~isempty(ik)
        j=j+1;
        header=[header C{k}(1:ik-1)];
        cj=[cj k];
    end
end
nc=length(cj);
V=zeros(0,nc);
j=0;
while 1
    j=j+1;
    line = fgetl(fid);
    if ~ischar(line)
        break
    end
    C = tools.strsplit(line,' ');
    C = C(cj);
    for i=1:nc
        V(j,i) = str2double(C{i});
    end
end
fclose(fid);

N=size(V,1);
param.value=[(1:N)' V];
param.name=header(2:end);
param.header=header;
param={param};

%%
function  p=mergeArg(p1,p2)
%mergeArg
%
%    p=mergeArg(p1,p2)



n1=length(p1);
n2=length(p2);
p=p1;
np=length(p);
for i=1:n2
    p2i=p2{i};
    if isfield(p2i,'header')
        testi=0;
        namei2=p2i.name;
        for j=1:n1
            p1j=p1{j};
            if isfield(p1j,'header')
                if strcmp(namei2,p1j.name)
                    p{j}=p2i;
                    testi=1;
                end
            else
                ifs=find(strcmp(p1j.name,namei2));
                if ~isempty(ifs)
                    p{j}.name(ifs)=[];
                    p{j}.value(ifs)=[];
                    np=np+1;
                    p{np}=p2i;
                    testi=1;
                end
            end
        end
        if testi==0
            np=np+1;
            p{np}=p2i;
        end
    else
        for k=1:length(p2i.name)
            namek2=p2i.name(k);
            testk=0;
            for j=1:n1
                p1i=p1{j};
                if isfield(p1i,'header')
                    if strcmp(namek2,p1i.name)
                        p{j}=struct('name',{namek2});
                        if isfield(p2i,'value')
                            p{j}.value=p2i.value;
                        end
                        if isfield(p2i,'time')
                            p{j}.time=p2i.time;
                        end
                        testk=1;
                    end
                else
                    ifs=find(strcmp(p1i.name,namek2));
                    if ~isempty(ifs)
                        p{j}.value(ifs)=p2i.value(k);
                        testk=1;
                    end
                end
            end
            if testk==0
                np=np+1;
                p{np}=struct('name',{namek2});
                if isfield(p2i,'value')
                    p{np}.value=p2i.value(k);
                end
                if isfield(p2i,'time')
                    p{np}.time=p2i.time;
                end
            end
        end
    end
end

function myparseModel(model_file, section, submodel_file)
%%myparseModel create a submodel_file corresponding to the specified sections of model_file
%
%    myparseModel(model_file, sections, submodel_file)
%       myparseModel create a submodel file corresponding to the specified sections of model_file
%
%       The specified sections of a model (model_file) are written in a new file (submodel_file)
%
%       sections :  a list of string containing a single name of the section we want to use to split the model.
%                   could be "POPULATION",  "OBSERVATION", "INDIVIDUAL", "COVARIATE"
%
%   Examples
%   --------
%       model_file    = "home/model.txt"
%       sections      =  {"COVARIATE"}
%       submodel_file = "home/submodel.txt"
%       myparseModel(model_file, sections, submodel_file)
%

sub_model = splitModel( model_file,section) ;
fichier   = fopen(submodel_file,'wt');
for i= 1 : length(sub_model)
    fprintf(fichier,'%s',sub_model{i});
    fprintf(fichier,'\n');
end
fclose(fichier);


function model = splitModel(file_model, section)

%%splitModel split a model file_model into a unique sub model corresponding to the specified section
%
%   sub_model = splitModel(file_model, sections)
%       splitModel split a model file_model into a unique submodel corresponding to the specified section
%
%       The extracted terms are returned in form of a cells of strings.
%       Each element of models is a line of the submodel
%
%       section :  a list of string containing a single  name of the section we want to use to split the model.
%                   could be "POPULATION", "COVARIATE","INDIVIDUAL", "OBSERVATION"
%   Examples
%   --------
%       file_model  = "home/model.txt"
%       sections    =  {"COVARIATE"}
%       model       = splitModel(file_model, sections)


if exist(file_model, 'file') ==2
    section     = [ '[' section ']'];
    fin_section = 0;
    % ouverture du fichier en lecture
    fid   = fopen(file_model);
    model = {};
    cpt   = 1;
    if fid == -1
        return
    end
    while 1
        line = fgetl(fid);
        if ~ischar(line) || fin_section==1
            % fin du fichier : on stoppe la lecture
            break
        end
        switch(line)
            case section
                model{cpt} =  line ;
                while 1
                    line = fgetl(fid);
                    cpt  = cpt + 1;
                    if ~ischar(line)
                        % fin du fichier : on stoppe la lecture
                        break
                    end
                    if (~isempty(strfind(line,'[')))
                        % fin du fichier : on stoppe la lecture
                        fin_section= 1 ;
                        break
                    end
                    model{cpt} =  line ;
                end
        end
    end
    
    fclose(fid);
else
    error(['file : ',file_model, ' does not exist']);
end

function inputList = getInputSection(model_file, section)
%   getInputSection extract the input list corresponding to the specified section of model_file
%
%    inputList = getInputSection(model_file, section)
%       getInputSection extract the input list corresponding to the specified section of model_file
%
%
%       section :  a string containing the name of the section we want to get the input list.
%                   could be "POPULATION",  "OBSERVATION", "INDIVIDUAL", "COVARIATE"
%
%   Examples
%   --------
%       model_file    = "home/model.txt"
%       section      =  {"INDIVIDUAL"}
%       inputList = getInputSection(model_file, section)
%       inputList = {"V_pop", "Cl_pop", "omega_V", "omega_Cl", "beta_V", "weight"}


%split a model file_model into multiple terms corresponding to the specified sections
sub_model = splitModel( model_file,section) ;
% extract the input list of the subsection
for i =1 : length(sub_model)
    if (~isempty(strfind(sub_model{i},'input')))
        inputList = sub_model{i};
        break;
    end
end
%   extraction des variables présentes entre les {}
%   Exemple :
%   "input = {V_pop, Cl_pop, omega_V, omega_Cl, beta_V, weight}"
%

%   extraction de la chaine présentes après "{" dans "input = {V_pop, Cl_pop, omega_V, omega_Cl, beta_V, weight}"
chaine1      = tools.strsplit(inputList,'{');
chaine1      = chaine1{2};
%   extraction de la chaine présentes avant "}" dans chaine1 = "V_pop, Cl_pop, omega_V, omega_Cl, beta_V, weight}"
chaine2      = tools.strsplit(chaine1,'}');
chaine2      = chaine2{1};
%   split en fonction de ","  dans chaine2 = "V_pop, Cl_pop, omega_V, omega_Cl, beta_V, weight"
inputList    = tools.strsplit(chaine2,',');











