function [outputIndividual, outputGroup, nbInds]= processing_output(output)
%processing_output 
%
%   [outputIndividual, outputGroup, nbInds]= processing_output(output)
%       The return results are : 
%           - outputIndividual : contains all population parameters  
%           - outputGroup      : contains all individual parameters  
%           - nbInds           : contains the number of differents individuals
%                                dealing with the output structure
%        
%
%   Examples
%   --------
%     [outputIndividual, outputGroup, nbInds]= processing_output(output)
% 
% output{1} =
%  
%      name: {'w_pop'  'V_pop'  'Cl_pop'}
%        id: [1 1 1]
%     label: {'none'  'none'  'none'}
% 
%  
%  
% output{2} =
%  
%      name: {'weight'}
%        id: 1
%     label: {'none'}
% 
%  
%  
% output{3} =
%  
%      name: {'V'  'Cl'}
%        id: [1 1]
%     label: {'none'  'none'}
% 
%  
%  
% output{4} =
%  
%      name: {'Cc'  'E'}
%      time: {[1x201 double]  [1x201 double]}
%        id: [1 1]
%     label: {'none'  'none'}
% 
%  
%  
% output{5} =
%  
%      name: {'y1'}
%      time: {[2 8 14 20 26 32 38 44 50 56]}
%        id: 1
%     label: {'none'}
% 
%  
%  
% output{6} =
%  
%      name: {'y2'}
%      time: {[6 14 22 30 38 46 54 62 70]}
%        id: 1
%     label: {'none'}
% 
%  
% outputIndividual
% 
% outputIndividual = 
% 
%      name: {{1x4 cell}}
%     label: {{1x4 cell}}
%      time: {{1x4 cell}}
%        id: 1
% 
% outputGroup
% 
% outputGroup = 
% 
%      name: {{1x6 cell}}
%     label: {{1x6 cell}}
%      time: {{1x4 cell}}
%        id: 1
% 
% nbInds
% 
% nbInds =
% 
%      1 


nb_output = length(output);
% format any single output cell
output    = cellfun(@formatOutput,output,'UniformOutput',false);

% get the number of individuals over all output cell
nbInds=zeros(1,length(output));
for i=1:nb_output
    nbInds(i) = length(unique(output{i}.id));
end
if (length(unique(nbInds))~=1)
    n_max=max(nbInds);
    for i=1 : length(output)
        if length(unique(output{i}.id))==1
            output{i}.name  = tools.replicateCellField(output{i}.name,n_max);
            output{i}.label = tools.replicateCellField(output{i}.label,n_max);
            output{i}.id    = 1:n_max;
            if isfield(output{i},'time')
                output{i}.time  = tools.replicateCellField(output{i}.time,n_max);
                output{i}.time  = repmat(output{i}.time',1,n_max);
            end
        end
    end
end

% get which output has a field "time"
outputTimes=zeros(1,length(output));
for i=1:length(output)
    if isfield(output{i}, 'time')
        outputTimes(i)=1;
    end
end

%% manage output for individual field
outputIndividual = {};
idx_times  = find(outputTimes==1);
if ~isempty(idx_times)
    outputIndividual.name   = tools.tocell(output{idx_times(1)}.name);
    outputIndividual.label  = tools.tocell(output{idx_times(1)}.label);
    outputIndividual.id     = output{idx_times(1)}.id;
    outputIndividual.time   = tools.tocell(output{idx_times(1)}.time);
    for i=2:length(idx_times)
        outputIndividual.name  = horzcat(outputIndividual.name,  tools.tocell(output{idx_times(i)}.name));
        outputIndividual.label = horzcat(outputIndividual.label,  tools.tocell(output{idx_times(i)}.label));
        outputIndividual.id    = [ outputIndividual.id output{idx_times(i)}.id];
        outputIndividual.time  = horzcat( outputIndividual.time, tools.tocell(output{idx_times(i)}.time));
    end
    
    u_id= unique(outputIndividual.id);
    for i=1 : length(u_id)
        idx= find(outputIndividual.id==u_id(i));
        temp.name{i}  = tools.tocell(outputIndividual.name{idx(1)});
        temp.label{i} = tools.tocell(outputIndividual.label{idx(1)});
        temp.time{i}     = tools.tocell(outputIndividual.time{idx(1),1});
        for j= 2 : length(idx)
            temp.name{i}  = horzcat(temp.name{i} ,tools.tocell(outputIndividual.name{idx(j)}));
            temp.label{i} = horzcat(temp.label{i}, tools.tocell(outputIndividual.label{idx(j)}));
            temp.time{i}  = horzcat(temp.time{i}, tools.tocell(outputIndividual.time{i,idx(j)}));
        end
    end
    temp.id  = u_id;
    outputIndividual = temp;
end


%% manage output for group field
outputGroup = {};
idx_times  = find(outputTimes==0);
if ~isempty(idx_times)
    outputGroup.name   = tools.tocell(output{idx_times(1)}.name);
    outputGroup.label  = tools.tocell(output{idx_times(1)}.label);
    outputGroup.id     = output{idx_times(1)}.id;
    for i=2:length(idx_times)
        outputGroup.name  = horzcat(outputGroup.name,  tools.tocell(output{idx_times(i)}.name));
        outputGroup.label = horzcat(outputGroup.label,  tools.tocell(output{idx_times(i)}.label));
        outputGroup.id    = [ outputGroup.id output{idx_times(i)}.id];
    end
    
    u_id= unique(outputGroup.id);
    for i=1 : length(u_id)
        idx= find(outputGroup.id==u_id(i));
        temp.name{i}  = tools.tocell(outputGroup.name{idx(1)});
        temp.label{i} = tools.tocell(outputGroup.label{idx(1)});
        for j= 2 : length(idx)
            temp.name{i}  = horzcat(temp.name{i} ,tools.tocell(outputGroup.name{idx(j)}));
            temp.label{i} = horzcat(temp.label{i}, tools.tocell(outputGroup.label{idx(j)}));
        end
    end
    temp.id  = u_id;
    outputGroup = temp;
end
nbInds = max(nbInds);
function output = formatOutput(output)
%formatOutput 
%
%   param = formatOutput(param)
%       checks fields in param structure and formats it by setting default field value
%            Set the good global format    
%        
%   Examples 1 
%   --------
%        outputOut = formatOutput(output)
% 
%     output = 
% 
%         name: {'Cc'}
%         time: [1x51 double]
% 
% 
% 
%     outputOut = 
% 
%          name: {'Cc'}
%          time: {[1x51 double]}
%            id: 1
%         label: {'none'}

defaultID      = tools.defaultFieldValue('id'); 
defaultLABEL   = tools.defaultFieldValue('label');

if isstruct(output)
    hasNAME   = isfield(output,'name');
    hasVALUE  = isfield(output,'value');
    hasHEADER = isfield(output,'header');
    if ~hasNAME
        error('output must have name as a field');      
    elseif ~hasVALUE && hasHEADER
        error('output with a header must have value as a field');
    elseif ~hasHEADER && hasVALUE
        error('output with a value must have header as a field');
    elseif hasHEADER && hasVALUE
        output = tools.addFieldsFromHeader(output);
    end

    % Expected field: name, id, TIME, label
    nbFields = 4;
    % creation de champs si necessaire
    output   = tools.setNonExistentField(output,'id',defaultID);
    output   = tools.setNonExistentField(output,'label',defaultLABEL);

    if length(fieldnames(output)) > nbFields
        error('presence of unexpected fields in output');
    end 
    
    % format if necessary
    dimID = length(output.id);
    output.name = tools.replicateCellField(output.name,dimID);
    dimNAME     = length(output.name);
    if isfield(output,'time')
        output.time = tools.tocell(output.time);
        output.time = repmat(output.time,1,dimNAME);
    end
    output.id    = tools.tovector(output.id,dimNAME);
    output.label = tools.replicateCellField(output.label,length(output.id));
    
else
    error('output must be a structure');
end
