function [multiParam, singleParam, nbInds]= processing_parameter(param)
%processing_parameter 
%
%   [multiParam, singleParam, nbInds] = processing_parameter(param)
%       The return results are : 
%           - singleParam : contains all population parameters  
%           - multiParam  : contains all individual parameters 
%           - nbInds           : contains the number of differents individuals
%                                dealing with the param structure
%        
%
%   Examples
%   --------
%     [multiParam, singleParam, nbInds] = processing_parameter(param)
% 
% param{1} =
%  
%      name: {'Cl_pop'  'omega_Cl'  'beta_Cl'  'ka'}
%     value: [0.2000 0.2000 0.7500 0.5000]
% 
%  
%  
% param{2} =
%  
%       name: {'weight'}
%     header: {'id'  'value'}
%      value: [5x2 double]
% 
%  
%  
% param{3} =
%  
%       name: {'V'}
%     header: {'id'  'value'}
%      value: [5x2 double]
% 
% 
% multiParam
% 
% multiParam = 
% 
%      name: {{1x2 cell}  {1x2 cell}  {1x2 cell}  {1x2 cell}  {1x2 cell}}
%     label: {{1x2 cell}  {1x2 cell}  {1x2 cell}  {1x2 cell}  {1x2 cell}}
%     value: {[60 1]  [85 0.5000]  [72 2]  [99 0.6000]  [68 0.8000]}
%        id: [1 2 3 4 5]
% 
% singleParam
% 
% singleParam = 
% 
%      name: {'Cl_pop'  'omega_Cl'  'beta_Cl'  'ka'}
%     value: [0.2000 0.2000 0.7500 0.5000]
% 
% nbInds
% 
% nbInds =
% 
%      1     5     5
% 
% 

singleParam = {};
multiParam  = {};

% format any single parameter cell
param = cellfun(@formatParameter,param,'UniformOutput',false);

% get the number of individuals over all param cell
nbInds=zeros(1,length(param));
for i=1:length(param)
    nbInds(i)=length(unique(param{i}.id));
end


%% manage population parameters
idx_paramPop  = find(nbInds==1);
if ~isempty(idx_paramPop)
    singleParam.name  = tools.tocell(param{idx_paramPop(1)}.name);
    singleParam.value = param{idx_paramPop(1)}.value;  
     for i=2:length(idx_paramPop)
          singleParam.name  = horzcat(singleParam.name,  tools.tocell(param{idx_paramPop(i)}.name));
          singleParam.value = [ singleParam.value param{idx_paramPop(i)}.value];
     end
end

%% manage individual parameters
idx_paramInd  = find(nbInds~=1) ;
if ~isempty(idx_paramInd)
    %check if the number of individuals are identicals in all parameter set
    tmp = nbInds(idx_paramInd(1));
    for i=2 : length(idx_paramInd)
        if tmp~=nbInds(idx_paramInd(i))
            error('number of individuals in param dismatch')
        end
    end        
    multiParam.name  = tools.tocell(param{idx_paramInd(1)}.name);
    multiParam.value = param{idx_paramInd(1)}.value;
    multiParam.label = tools.tocell(param{idx_paramInd(1)}.label);
    multiParam.id    = param{idx_paramInd(1)}.id;

     for i=2:length(idx_paramInd)
          multiParam.name   = horzcat(multiParam.name,  tools.tocell(param{idx_paramInd(i)}.name));
          multiParam.label  = horzcat(multiParam.label, tools.tocell(param{idx_paramInd(i)}.label));
          multiParam.value  = [multiParam.value param{idx_paramInd(i)}.value];
          multiParam.id     = [multiParam.id ;param{idx_paramInd(i)}.id];
     end
     multiParam.id          = unique(multiParam.id )';
     for i=1 : length(multiParam.id)
         temp.name{i}  = multiParam.name;
         temp.label{i} = multiParam.label;
         temp.value{i} = multiParam.value(i,:);
     end
     temp.id    = multiParam.id;
     multiParam = temp;
end
nbInds=max(nbInds);

function param = formatParameter(param)
%formatParameter 
%
%   param = formatParameter(param)
%       checks fields in param structure and formats it by setting default field value
%            Set the good global format    
%        
%   Examples 1 
%   --------
%       paramOut = formatParameter(param)
% 
%       param = 
% 
%         name: {'Cl_pop'  'omega_Cl'  'beta_Cl'  'ka'}
%         value: [0.2000 0.2000 0.7500 0.5000]
%       
%       paramOut = 
% 
%         name: {'Cl_pop'  'omega_Cl'  'beta_Cl'  'ka'}
%         value: [0.2000 0.2000 0.7500 0.5000]
%         id: 1
%         label: 'none'


defaultlabel   = tools.defaultFieldValue('label'); 

if isstruct(param)
    hasNAME   = isfield(param,'name');
    hasVALUE  = isfield(param,'value');
    hasHEADER = isfield(param,'header');
    if (~hasNAME &&  ~hasHEADER)
        error('param must have name as a field');  
    elseif ~hasVALUE
        error('param must have value as a field');
    elseif hasHEADER
        param = tools.addFieldsFromHeader(param);
    end
    % add fields with default values if necessary
    param = tools.setNonExistentField(param,'id',1:size(param.value,1));
    param = tools.setNonExistentField(param,'label',defaultlabel);
    if length(param.id)>1
        if size(param.value,1)==1
            param.value=param.value';
        end
    end
    % Expected field: name, id, label, value
    nbFields = 4;
    if length(fieldnames(param)) > nbFields
        error('presence of unexpected fields in param');
    end 
else
    error('param must be a structure');
end
