classdef tools
   methods(Static)
      function value = defaultFieldValue(field)
        % defaultFieldValue
        %
        %    value = defaultFieldValue(field)
        %       return the defaut value of the input field
        %
        %   Examples
        %   --------
        %       value = defaultFieldValue("size")
        %       value =  1
        %
        switch(field)
            case 'amount'
                value = 0; % a confirmer
            case 'id'
                value = 1;
            case 'label'
                value = 'none';
            case 'name'
                value = 'none';
            case 'rate'
                value = 0;
            case 'size'
                value = 1;
            case 'type'
                value = 1;
            otherwise
                error('unknown field');
        end
      end
    
      function vector=tovector(arg,dim)
        % tovector 
        %
        %    vector=tovector(arg,dim)
        %      return a vector of length dim and value arg
        %
        %   Examples
        %   --------
        %       vector = tovector(arg,dim)
        %       arg=2, dim=4, 
        %       vector = [2 2 2 2]

        %           
        if isnumeric(arg)
            if isscalar(arg) 
                % if  arg is a scalar
                vector = repmat(arg,1,dim);
            elseif ~isvector(arg)
                error('arg must be a vector');
            elseif length(arg)~=dim
                % if arg is a vector and dim is a vector of different size 
                error('arg cannot be a vector of size different from dim');
            else
                % if arg is a vector and size(dim) = size(arg) 
                % (1 ou length(arg))
                vector= arg; 
            end    
        else
            % if "arg" is not numerical (cells). the input argument arg is
            % returned withour any modifications
            vector= arg; 
        end
      end

      function structure = setNonExistentField(structure,field,value)
        % setNonExistentField 
        %
        %    structure = setNonExistentField(structure,field,value)
        %       take a structure and set a field and its corresponding value.
        %
        %   Examples
        %   --------
        %     structureOut = setNonExistentField(structure,field,value)
        % 
        %     structure = 
        %         name: {'ka'  'V'  'Cl'}
        %         value: [0.5000 8 1]
        %
        %     field = "id"
        %     value = 1
        % 
        %      structureOut = 
        %          name: {'ka'  'V'  'Cl'}
        %         value: [0.5000 8 1]
        %            id: 1    

        if ~isfield(structure,field)
            structure.(field) = value;
        end
      end

      function c=tocell(s)
        % tocell 
        %
        %     c=tocell(s)
        %       convert s to cell

        if ~iscell(s)
            c = {s};
        else
            c=s;
        end

        for k=1:length(c)
            if isfield(c{k},'name')
                if ~iscell(c{k}.name)
                    c{k}.name={c{k}.name};
                end
            end
        end
      end

      function structure_field = replicateCellField(structure_field,dim)
        % replicateCellField
        %
        %   structure_field = replicateCellField(structure_field,dim)
        %       Replique dim fois une structure
        %
        %   Exemple :
        %       structure_fieldOut = replicateCellField('A',4)
        %       structure_fieldOut =   'A'    'A'    'A'    'A'
        %

        structure_field = tools.tocell(structure_field);
        if length(structure_field)==1
            for i=2: dim
                structure_field{i} = structure_field{1};
            end
        end
      end
      
      function structure = addFieldsFromHeader(structure)
        % addFieldsFromHeader 
        %
        %   structure = addFieldsFromHeader(structure)
        %       take a header list on a structure and create a structure of header
        %       fields. Each field is composed by a cell array per individual. 
        %       The refered individuals are in id field  
        %
        %   Examples
        %   --------
        %       structureOut = addFieldsFromHeader(structure)
        %       structure =  
        %           header : {'id'  'time'  'amount'  'type'  'rate'}
        %           value  : [6x5 double]
        %       structureOut =
        %       id     :  [1 2 3]
        %       time   : {3x1 cell}
        %       amount : {3x1 cell}
        %       type   : {3x1 cell}
        %       rate   : {3x1 cell}


        header = structure.header;
        value  = structure.value;

        structure = rmfield(structure,'header');
        structure = rmfield(structure,'value');


        %% only if name is a field
        couples=[];
        if isfield(structure,'name')
            names = tools.tocell(structure.name);
            for i=1: length(names)
                for j=1: length(header)
                    if isequal(names{i},header{j})
                        couples=[couples ;[i j]];
                    end
                end
            end
        end

        %% create fields directly from header and value
        if isempty(couples)
            for i=1:length(header)
                structure.(header{i}) = value(:,i)';
            end

        else
            idx_id=find(strcmp(header,'id'));
            structure.id = value(:,idx_id);
            structure.value=zeros(length(structure.id), size(couples,1));
            for j=1 : size(couples,1)
                structure.value(:,j)= value(:,couples(j,2));
            end
        end


        %% Only if "id" is a field
        %  
        if isfield(structure,'id')
            ids = unique(structure.id);
            if size(ids,1)>1
                ids=ids';
            end
            structure = tools.splitField(structure,'time',ids);
            structure = tools.splitField(structure,'amount',ids);
            structure = tools.splitField(structure,'rate',ids);
            structure = tools.splitField(structure,'type',ids);

            structure.id = ids;
            %
        else
            error('id must be a field');
        end
      end


        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %  splitField function 
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function structure = splitField(structure,field,ids)
            % splitField 
            %
            %   structure = splitField(structure,field,ids)
            %   splitField split a field in a structure according to ids  
            %   field. It create a cell per individual. 
            %    
            %
            %   Examples
            %   --------
            %   structureOut = splitField(structure,"time",[1 2 3])
            %       structure = 
            % 
            %         id: [1 1 2 2 3 3]
            %       time: [0 12 0 12 0 12]
            %     amount: [5 10 5 10 10 5]
            %       type: [1 1 2 2 1 2]
            %       rate: [0 0 0 0 0 2]
            %       
            %     structureOut = 
            % 
            %       id    : [1 1 2 2 3 3]
            %       time  : {3x1 cell}
            %       amount: [5 10 5 10 10 5]
            %       type  : [1 1 2 2 1 2]
            %       rate  : [0 0 0 0 0 2]
            %       
            %     structureOut.time
            % 
            %     [1x2 double]
            %     [1x2 double]
            %     [1x2 double]

            if isfield(structure,field)
                c = cell(length(ids),1);
                for i=1:length(ids)
                   c{i} = structure.(field)(structure.id==ids(i));
                end
                structure.(field) = c; 
            end
        end
        
        function terms = strsplit(s, delimiter)
        %STRSPLIT Splits a string into multiple terms
        %
        %   terms = strsplit(s)
        %       splits the string s into multiple terms that are separated by
        %       white spaces (white spaces also include tab and newline).
        %
        %       The extracted terms are returned in form of a cell array of
        %       strings.
        %
        %   terms = strsplit(s, delimiter)
        %       splits the string s into multiple terms that are separated by
        %       the specified delimiter. 
        %   
        %   Remarks
        %   -------
        %       - Note that the spaces surrounding the delimiter are considered
        %         part of the delimiter, and thus removed from the extracted
        %         terms.
        %
        %       - If there are two consecutive non-whitespace delimiters, it is
        %         regarded that there is an empty-string term between them.         
        %
        %   Examples
        %   --------
        %       % extract the words delimited by white spaces
        %       ts = strsplit('I am using MATLAB');
        %       ts <- {'I', 'am', 'using', 'MATLAB'}
        %
        %       % split operands delimited by '+'
        %       ts = strsplit('1+2+3+4', '+');
        %       ts <- {'1', '2', '3', '4'}
        %
        %       % It still works if there are spaces surrounding the delimiter
        %       ts = strsplit('1 + 2 + 3 + 4', '+');
        %       ts <- {'1', '2', '3', '4'}
        %
        %       % Consecutive delimiters results in empty terms
        %       ts = strsplit('C,Java, C++ ,, Python, MATLAB', ',');
        %       ts <- {'C', 'Java', 'C++', '', 'Python', 'MATLAB'}
        %
        %       % When no delimiter is presented, the entire string is considered
        %       % as a single term
        %       ts = strsplit('YouAndMe');
        %       ts <- {'YouAndMe'}
        %

        %   History
        %   -------
        %       - Created by Dahua Lin, on Oct 9, 2008
        %

        %% parse and verify input arguments

            assert(ischar(s) && ndims(s) == 2 && size(s,1) <= 1, ...
                'strsplit:invalidarg', ...
                'The first input argument should be a char string.');

            if nargin < 2
                by_space = true;
            else
                d = delimiter;
                assert(ischar(d) && ndims(d) == 2 && size(d,1) == 1 && ~isempty(d), ...
                    'strsplit:invalidarg', ...
                    'The delimiter should be a non-empty char string.');

                d = strtrim(d);
                by_space = isempty(d);
            end

            %% main

            s = strtrim(s);

            if by_space
                w = isspace(s);            
                if any(w)
                    % decide the positions of terms        
                    dw = diff(w);
                    sp = [1, find(dw == -1) + 1];     % start positions of terms
                    ep = [find(dw == 1), length(s)];  % end positions of terms

                    % extract the terms        
                    nt = numel(sp);
                    terms = cell(1, nt);
                    for i = 1 : nt
                        terms{i} = s(sp(i):ep(i));
                    end                
                else
                    terms = {s};
                end

            else    
                p = strfind(s, d);
                if ~isempty(p)        
                    % extract the terms        
                    nt = numel(p) + 1;
                    terms = cell(1, nt);
                    sp = 1;
                    dl = length(delimiter);
                    for i = 1 : nt-1
                        terms{i} = strtrim(s(sp:p(i)-1));
                        sp = p(i) + dl;
                    end         
                    terms{nt} = strtrim(s(sp:end));
                else
                    terms = {s};
                end        
            end
        end
    end
end